/*
 * wstat.c
 * prints statistics about weights and compressed representations when learning begins
 *
 * version: 1.0
 * (c) Michal Pokorny, apr 2006
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lraam.h"

lraam_t * raam;
set_t * dataset;
set_t * termset;
extern int ARITY;

// main
// -----------------------------------------------------------------------------
int
main(int argc, char **argv) {

    error_t err;
    int i,ii,j,k,a;
    FILE * f, * g, *h;
    double eta;
    int dim;

    if (argc < 2) {
	fprintf(stderr, "USAGE: ./%s <weight>\n", argv[0]);
    }
    sscanf(argv[1], "%lf", &eta);

    // loads experiment
    err = experiment_load(stdin, &raam, &dataset, &termset);
    if (err != ERR_OK) {
	fprintf(stderr, "ERROR %d: loading experiment failed\n", err);
	exit(1);
    }
    
    f = fopen("tmp.compress", "w");
    g = fopen("tmp.output", "w");
    h = fopen("tmp.delta", "w");

    dim = raam->dimension;

    // for each structure compute its compressed representation
    for (i=0; i<dataset->no_of_objects; i++) {
	if (descr_length(dataset->objects[i]->structure.description) != ARITY + 2) {
	    fprintf(stderr, "ERROR %d[%d]: unsupported depth of structure\n", i, descr_length(dataset->objects[i]->structure.description));
	    exit(1);
	}
	for (j=0; j<ARITY; j++) {
	    k = dataset->objects[i]->structure.description[j+2];
	    if (k < 0 || k >= termset->no_of_objects) {
		fprintf(stderr, "ERROR %d[%d]=%d: unsupported structure\n", i, j+2, k);
		exit(1);
	    }
	    layer_set_node(raam->input_layer, termset->objects[k]->representation, j);
	}
	lraam_encode(raam);
	lraam_decode(raam);
	vector_fprintf(f, raam->compression_layer, raam->dimension, "");
	layer_fprintf(g, raam->output_layer, raam->dimension, "", "");
	if (eta == 0) continue;
	for (a=0; a<ARITY; a++)
	for (ii=0; ii<dim; ii++)
	for (j=0; j<dim; j++)
	    value_fprintf(h, eta * raam->compression_layer[ii] * (raam->input_layer[a*dim+j] - raam->output_layer[a*dim+j]));
	    fprintf(h, "\n");
    }

    // releasing memory
    lraam_free(raam);
    set_free(dataset);
    set_free(termset);
    
    fclose(f);
    fclose(g);
    fclose(h);

    return 0;
}
